# =====================================================================
#  Written by and copyright Carlo Strozzi <carlos@linux.it>.
#
#  list2form.awk: default list-to-form filter for CSA.
#  Copyright (C) 2001-2002 Carlo Strozzi <carlos@linux.it>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# Notes:
#
# Default filter file for transposing a single-record table 'list' file
# to a response form.
#
# ChangeLog:
#
# * Development code
#   2002-11-05 Carlo Strozzi <carlos@linux.it>
#
# =====================================================================

/^$/ { c_values[s_num_cols] = s_hold_space }

/^[a-zA-Z_]/ {						# New column.
   if (s_prev_record == NULL) {
      delete c_names
      delete c_values
      s_num_cols = 0
      s_hold_space = NULL
   }
   if (s_hold_space != NULL) {				# Flush hold-space.
      c_values[s_num_cols] = s_hold_space
      s_hold_space = NULL
   }
   c_names[++s_num_cols] = $1
   s_hold_space = $0
   sub(/^[^\t ]+([\t ]+|$)/, NULL, s_hold_space)
}

/^\t/ {
   s_hold_space = s_hold_space "\n\t" substr($0,2)	# Continuation line.
}

{ s_prev_record = $0 }

END {
   printf("\n")					# Print start separator.
   for (i=1; i <= s_num_cols; i++) {
      print c_names[i], c_values[i]
   }
   printf("\n")					# Print end separator.
}

# EOF
