# =====================================================================
# csa-buffer.awk: CSA final HTTP response processor/parser.
#
# Copyright (c) 2007,2009,2010 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================

# =====================================================================
# void _buffer(string line)
#
# Parse input for special (i.e. Wiki-like) markup and turn i into the
# corresponding XHTML markup. The final result is made available in the
# _OUT global array.
# =====================================================================

function _buffer(line,				a,i,j) {

   # Turn "soft" line breaks back to hard ones to make it possible
   # for the wiki stream to be parsed. Account for the fact that
   # split() returns '0' on empty lines, including the one that
   # separates the header from the body of the HTTP response.

   if (!(i=split(line,a,/<csa:br *\/?>/))) i++

   # This is just for testing. A lot more processing will occur in
   # real-world situations before the final parsed output ends up
   # into _OUT[].

   for (j=1; j<=i; j++) _BUFFER[++_BUFFER[0]] = _OUT[++_OUT[0]] = a[j]
}

# =====================================================================

{ _buffer($0) }				# populate _OUT[]

END {

  s = S = _NULL				# init local vars.

  for (i=1; i<=_OUT[0]; i++) s = s _OUT[i] "\n"

  # Sequences of one or more hyphens are theoretically against the
  # specs if they occur inside XML comments. This may or may not be
  # a problem, depending on how strict is the XML implementation in
  # clients. MSIE seems not to bother, while FF and possibly others
  # seem to be more picky, so I prefer to void such malformed comments
  # for everyone, just in case.

  gsub(/<!--[^>]*--[^>]*-->/,"<!-- malformed comment removed by CSA -->",s)

  if (!(i=length(s))) exit(1)		# account for CSA_TPLWB.

  if (_csa("cpi") == _TRUE) {
     s = _parsecpi(s)
     i = length(s)
  }

  # w.bloggar (www.wbloggar.com) gets confused by URLs which 'href'
  # attribute value is enclosed in single-quotes so I need to turn them
  # into double-quotes. Add tests for more buggy user agents here, along
  # with the necessary support code and workarounds.

  if (ENVIRON["HTTP_USER_AGENT"] ~ /w\.bloggar/) {
     gsub(/&#39;/,"\001",s)
     while (_extract(s,\
        "[ \t\n\r]+[hH][rR][eE][fF][ \t\n\r]*=[ \t\n\r]*\001[^\001]+\001")) {
        gsub(/\001/,"\\&#34;",_RMATCH)
        S = S _RLMATCH _RMATCH
     }
     S = S _RRMATCH
     gsub(/\001/,"\\&#39;",S)
     s = S
  }

  # Next line is a hack to make my own patched thttpd(8) handle the HEAD
  # method correctly (stock thttpd supports HEAD only with static pages,
  # not with CGI programs). This hack is unnecessary with more common
  # HTTP servers, like Apache.

  if (ENVIRON["REQUEST_METHOD"] == "HEAD") s = _NULL

  printf("Content-Length: %d\n%s\n",--i,substr(s,1,i))
}

# =====================================================================
# End of csa-buffer.awk
# =====================================================================
