# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2007,2008,2009 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# void _html(int what, anytype arg1, anytype arg2)
#
# Perform misc. on-the-fly manipulations on the [X]HTML response stream
# while serving it to the client browser. The meaning and expected
# contents of "arg1" and "arg2" depend on the content of "what".
#
# At the moment the only function implemented is the toggling of
# visibility between selected response template sections, namely those
# comprised between the specified "arg1" and "arg2" tags included.
# Nested conditionals are not supported, and if the specified tags are
# not placed properly in the template structure the resulting response
# may well no longer parse as proper [X]HTML.
# =====================================================================

function _html(what,arg1,arg2) {

  if (what == "show" || what == "hide") {

     if (arg1 == _NULL || arg2 == _NULL) return _EINVAL

     if (what == "show") {
	_response(arg1,_NULL)
	_response(arg2,_NULL)
     }

     else {

	# If CPI is enabled then then it it the safest method to
	# handle contitional sections in [X]HTML responses.

	if (_csa("cpi") == _TRUE) {
	   _response(arg1,"(::DEL:)")
	   _response(arg2,"(:DEL::)")
	}
	else {
	   # With XHTML 1.1 the <script> tag may not be put anywhere
	   # in the document; for instance, it may not appear between
	   # a '<ul>' and its first '<li>'. The only thing that seems to
	   # always work are comments, but this means that a conditional
	   # block may not contain comments itself, or things will break.
	   # Short of a better solution, I'll use comments for now.
	   # A better approach, not yet implmented, would be to use
	   # '<csa:comment> ... </csa:comment>' tags and have them
	   # filtered-out by by the final CSA response buffering.

	   _response(arg1,"<!-- ")
	   _response(arg2," -->")
	}
     }
  }
}

