# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2006 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# boolean _isdate(string iso_date, [boolean short])
#
# Takes a date in ISO format and returns _TRUE if it is a valid (i.e.
# existing) date, or _FALSE otherwise. The input date is expected to
# be in "YYYY-mm-dd [HH:MM[:SS]]" format, or _FALSE will be returned.
# The function tries to be "resilient" as to what separator characters
# are used in the input date, but it is not bullet-proof and the input
# should preferably be separated as shown. If  'short' is set to '1'
# then even a date composed only by "YYYY-mm-dd" is considered valid.
# =====================================================================

function _isdate(date,short,					a,i) {

  if (split(date,a," ") < 2) {
     if (short) a[2] = "00:00:00"
     else return _FALSE
  }

  # Add seconds if only HH:MM are present.
  if (a[2] ~ /^[0-2][0-9]:[0-5][0-9]$/) a[2] = a[2] ":00"

  date = a[1] " " a[2]

  if ((i=split(date,a,/[-\/ :.]/)) < 3 || i > 6) return _FALSE

  gsub(/[^0-9]/,"-",date)

  i = _gregorian(_julian(a[1],a[2],a[3],a[4],a[5],a[6]),"i")
  gsub(/[^0-9]/,"-",i)

  if (i != date) return _FALSE

  return _TRUE
}
