/* csa-server-push: take a list of files on the command line and print
   their content to stdout in MIME multipart format, in an endless loop,
   sleeping 1/60th of a second between each file.

   Copyright (c) 2003,2006 Carlo Strozzi

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 dated June, 1991.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   ********************************************************************

   Usage:  csa-server-push [-m|--mime-type 'type'] [-l|--loops 'n'] \
		[-b|--mime-boundary 'string'] [-l|--loops 'n'] \
		[-s|--sleep 'u'] file [file ...]

   See http://wp.netscape.com/assist/net_sites/pushpull.html for more info.

   $Id: csa-server-push.c 43 2007-09-24 08:30:21Z carlo $

*/

static char Psz_RCS[] = "$Date: 2006/03/10 21:36:17 $";
static char mime_type[] = "application/octet-stream";
static char mime_boundary[] = "CSA_MIME_BOUNDARY";

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

static int eusage(void) {
  fprintf(stderr, "Usage: csa-server-push [options] file [file ...]\n");
  return 1;
}

int main(int argc, char **argv) {

    int i=0, min_args=2, mime_t=0, mime_b=0, loops=0, delay=0;
    char c;

    FILE *fp;			/* Input file */

    while (++i < argc && *argv[i] == '-') {
       min_args++;
       if (!strcmp(argv[i], "-m") ||
		!strcmp(argv[i], "--mime-type")) {
	  if (++i >= argc || *argv[i] == '-') exit(eusage());
	  min_args++;
	  mime_t = i;
       }
       else if (!strcmp(argv[i], "-b") ||
		!strcmp(argv[i], "--mime-boundary")) {
	  if (++i >= argc) exit(eusage());
	  min_args++;
	  mime_b = i;
       }
       else if (!strcmp(argv[i], "-l") ||
		!strcmp(argv[i], "--loops")) {
	  if (++i >= argc || *argv[i] == '-') exit(eusage());
	  min_args++;
	  loops = atoi(argv[i]);
       }
       else if (!strcmp(argv[i], "-s") ||
		!strcmp(argv[i], "--sleep")) {
	  if (++i >= argc || *argv[i] == '-') exit(eusage());
	  min_args++;
	  delay = atoi(argv[i]);
       }
    }

    if (argc < min_args) exit(eusage());

    /* print MIME header */
    printf("Content-Type: multipart/x-mixed-replace;");

    if (mime_b) printf("boundary=%s\n\n", argv[mime_b]);
    else printf("boundary=%s\n\n", getenv(mime_boundary));

    i=0;

    while (1) {

       if (++i >= argc) i=1;

       if (delay) usleep(delay);

       /* Open one file for read. Ignore open errors, as they may happen
	  while the input image files are being rewritten+renamed.
	  At worst we will skip one frame, but who cares. */

       if ((fp = fopen(argv[i], "r")) == NULL) continue;

       /* print MIME boundary */
       if (mime_b) printf("%s\n", argv[mime_b]);
       else printf("%s\n", getenv(mime_boundary));

       /* print MIME part type */
       if (mime_t) printf("Content-Type: %s\n\n", argv[mime_t]);
       else printf("Content-Type: %s\n\n", mime_type);

       while (1) {
	  c = fgetc(fp);
	  if (feof(fp)) break;
	  putchar(c);
       }
       fclose(fp);
       putchar('\n');		/* begin new MIME header */

       /* fflush(stdout); */

       if (loops) {
	  if (--loops <= 0) exit(0);
       }
    }

    /* print MIME footer */
    if (mime_b) printf("%s--\n", argv[mime_b]);
    else printf("%s--\n", getenv(mime_boundary));

    exit(0);
}

/* EOF */
